/*============================================================================
  File:     UseDocRepository.sql

  Summary:  Sparse columns basic perf test

  Date:     August 2008

  SQL Server Version: 10.0.1600.22 (SQL Server 2008 RTM)
------------------------------------------------------------------------------
  Written by Kimberly L. Tripp & Paul S. Randal, SQLskills.com

  For more scripts and sample code, check out http://www.SQLskills.com

  This script is intended as a supplement to the SQL Server 2008 Jumpstart or
  Metro training.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

-- Two tables have been pre-created
--		NonSparseDocRepository and SparseDocRepository
-- Each has 100000 rows of document metadata. There are a
-- few common column values for each document (e.g. name,
-- type) and then 20 columns that are distinct to each
-- type - stored in columns c20-c39 for doc-type 1,
-- c40-c59 for doc-type 2 etc
-- Although these are stored as integers, pretend they
-- could be various types.
-- Each table has a clustered index on DocID.

USE SparseColumnsTest;
GO

-- Examine the row lengths and page counts
-- Explain about SAMPLED scans while its running
SELECT [avg_record_size_in_bytes], [page_count], *
	FROM sys.dm_db_index_physical_stats (
	DB_ID ('SparseColumnsTest'),
	OBJECT_ID ('NonSparseDocRepository'),
	NULL, NULL, 'SAMPLED');
SELECT [avg_record_size_in_bytes], [page_count], *
	FROM sys.dm_db_index_physical_stats (
	DB_ID ('SparseColumnsTest'),
	OBJECT_ID ('SparseDocRepository'),
	NULL, NULL, 'SAMPLED');
GO

-- empty the buffer pool
DBCC DROPCLEANBUFFERS;
GO

-- Random selects - trade-off extra space for non-sparse
-- against extra time to crack sparse columns
SELECT COUNT (*) FROM NonSparseDocRepository WHERE
	c741 > 0 OR c334 > 0 OR c122 > 0 OR c904 > 0;
GO
-- Time: approx 40 seconds

DBCC DROPCLEANBUFFERS;
GO

SELECT COUNT (*) FROM SparseDocRepository WHERE
	c741 > 0 OR c334 > 0 OR c122 > 0 OR c904 > 0;
GO

-- Basically the huge drop in page count is the deciding
-- factor