-- Testing table/row structure on conversion from 0 sparse columns to having sparse columns...

DROP TABLE Test
-- Trying to get a table to fail when converting to SPARSE
CREATE TABLE Test
(
    c1  int identity,
    c2  datetime,
    c3  datetime,
    c4  char(8000)
)
go
INSERT Test VALUES (getdate(), getdate(), 'this is a test')
go 10

ALTER TABLE Test
ALTER COLUMN c4 char(8000) SPARSE
go

--Msg 511, Level 16, State 1, Line 1
--Cannot create a row of size 16039 which is greater than the allowable maximum row size of 8060.

DROP TABLE Test2
-- Trying to get a table to fail when converting to SPARSE
CREATE TABLE Test2
(
    c1  int identity,
    c2  datetime,
    c3  datetime,
    c4  char(6000)
)
go
INSERT Test2 VALUES (getdate(), getdate(), 'this is a test')
go 10

ALTER TABLE Test2
ALTER COLUMN c4 char(6000) SPARSE
go

--Msg 511, Level 16, State 1, Line 2
--Cannot create a row of size 12039 which is greater than the allowable maximum row size of 8060.

DROP TABLE Test3
-- Trying to get a table to fail when converting to SPARSE
CREATE TABLE Test3
(
    c1  int identity,
    c2  datetime,
    c3  datetime,
    c4  char(5000)
)
go
INSERT Test3 VALUES (getdate(), getdate(), 'this is a test')
go 10

ALTER TABLE Test3
ALTER COLUMN c4 char(5000) SPARSE
go

--Msg 511, Level 16, State 1, Line 2
--Cannot create a row of size 10039 which is greater than the allowable maximum row size of 8060.

DROP TABLE Test4
-- Trying to get a table to fail when converting to SPARSE
CREATE TABLE Test4
(
    c1  int identity,
    c2  datetime,
    c3  datetime,
    c4  char(4030)
)
go
INSERT Test4 VALUES (getdate(), getdate(), 'this is a test')
go 10

ALTER TABLE Test4
ALTER COLUMN c4 char(4030) SPARSE
go

--Msg 511, Level 16, State 1, Line 2
--Cannot create a row of size 8099 which is greater than the allowable maximum row size of 8060.

-------------------------------------------------------------
DROP TABLE Test5
-- Trying to get a table to fail when converting to SPARSE
CREATE TABLE Test5
(
    c1  int identity,
    c2  datetime,
    c3  datetime,
    c4  char(4000)
)
go
INSERT Test5 VALUES (getdate(), getdate(), 'this is a test')
go 10

SELECT * FROM Test5
go

ALTER TABLE Test5
ADD SparseColumns XML COLUMN_SET FOR ALL_SPARSE_COLUMNS
go

SELECT * FROM Test5
go

ALTER TABLE Test5
ALTER COLUMN c4 char(4000) SPARSE
go

SELECT * FROM Test5
go

-------------------------------------------------------------
DROP TABLE Test5
-- Trying to get a table to fail when converting to SPARSE
CREATE TABLE Test5
(
    c1  int identity,
    c2  datetime,
    c3  datetime,
    c4  char(4000)
)
go
INSERT Test5 VALUES (getdate(), getdate(), 'this is a test')
go 10

SELECT * FROM Test5
go

ALTER TABLE Test5
ALTER COLUMN c4 char(4000) SPARSE
go

SELECT * FROM Test5
go

ALTER TABLE Test5
ADD SparseColumns XML COLUMN_SET FOR ALL_SPARSE_COLUMNS
go

--Msg 1734, Level 16, State 1, Line 1
--Cannot create the sparse column set 'SparseColumns' in the table 'Test5' because the table already contains one or more sparse columns. A sparse column set cannot be added to a table if the table contains a sparse column.

SELECT * FROM Test5
go
-------------------------------------------------------------
DROP TABLE Test5
-- Trying to get a table to fail when converting to SPARSE
CREATE TABLE Test5
(
    c1  int identity,
    c2  datetime,
    c3  datetime,
    c4  char(4000)
)
go
INSERT Test5 VALUES (getdate(), getdate(), 'this is a test')
go 10

SELECT * FROM Test5
go

SELECT * FROM sys.dm_db_index_physical_stats(db_id(), object_id('Test5'), null, null, 'detailed')
--min_record_size_in_bytes	max_record_size_in_bytes	avg_record_size_in_bytes
--4027	                    4027	                    4027

ALTER TABLE Test5
ALTER COLUMN c4 char(4000) SPARSE
go

SELECT * FROM sys.dm_db_index_physical_stats(db_id(), object_id('Test5'), null, null, 'detailed')
--min_record_size_in_bytes	max_record_size_in_bytes	avg_record_size_in_bytes
--9	                        8051	                    5366.333

--- why does this last one work?
-- the column DOES get duplicated internally as they convert the row
-- 4+12+12+overhead+oldvalue+sparsecolumnoverhead+sparsevalue <= 8060

-- TWO PROBLEMS:
-- 1 - you can't add an XML column set unless you add that first (so, I guess that's not too bad?!)...
-- 2 - this internal "conversion" is WEIRD, CONFUSING and UNDESIRABLE!!!! 

-- What's a better approach?
-- DON'T DO THAT?! ;-)

DROP TABLE Test5
-- Trying to get a table to fail when converting to SPARSE
CREATE TABLE Test5
(
    c1  int identity,
    c2  datetime,
    c3  datetime,
    c4  char(4000)
)
go
INSERT Test5 VALUES (getdate(), getdate(), 'this is a test')
go 10

SELECT * FROM Test5
go

CREATE Table Test6
(
    c1  int identity,
    c2  datetime,
    c3  datetime,
    SparseColumns XML COLUMN_SET FOR ALL_SPARSE_COLUMNS,
    c4  char(4000) SPARSE
)
go

INSERT Test6 (c2, c3, c4)
    SELECT c2, c3, c4 FROM Test5
go

SELECT * FROM sys.dm_db_index_physical_stats(db_id(), object_id('Test6'), null, null, 'detailed')
--min_record_size_in_bytes	max_record_size_in_bytes	avg_record_size_in_bytes
--4039	                    4039	                    4039